@@ -4,12 +4,11 @@ or in a Jupyter Notebook to organize and interact with your unstructured data.
 """
 import concurrent.futures
 import json
-import os
 from typing import Dict, List, Optional
 
 import numpy as np
-import pydantic
 import requests
+from loguru import logger
 from pydantic import BaseModel, Field
 from tqdm import tqdm
 from wonderwords import RandomWord
@@ -53,7 +52,7 @@ class AtlasClient:
                 headers=self.header,
             )
             if not response.status_code == 200:
-                print("Your authorization token is no longer valid.")
+                logger.info("Your authorization token is no longer valid.")
         else:
             raise ValueError(
                 "Could not find an authorization token. Run `nomic login` to authorize this client with the Nomic API."
@@ -65,7 +64,7 @@ class AtlasClient:
             headers=self.header,
         )
         if not response.status_code == 200:
-            raise ValueError("Your authorization token is no longer valid.")
+            raise ValueError("Your authorization token is no longer valid. Run `nomic login` to obtain a new one.")
 
         return response.json()
 
@@ -81,16 +80,16 @@ class AtlasClient:
         '''
         Creates an Atlas project. Atlas projects store data (text, embeddings, etc) that you can organize by building indices.
 
-        Args:
-            project_name: The name of the project.
-            description: A description for the project.
-            unique_id_field: The field that uniquely identifies each datum. If a datum does not contain this field, it will be added and assigned a random unique ID.
-            modality: The data modality of this project. Currently, Atlas supports either `text` or `embedding` modality projects.
-            organization_name: The name of the organization to create this project under. You must be a member of the organization with appropriate permissions. If not specified, defaults to your user accounts default organization.
-            is_public: Should this project be publicly accessible for viewing (read only). If False, only members of your Nomic organization can view.
+        **Parameters:**
 
-        Returns:
-            project_id on success.
+        * **project_name** - The name of the project.
+        * **description** - A description for the project.
+        * **unique_id_field** - The field that uniquely identifies each datum. If a datum does not contain this field, it will be added and assigned a random unique ID.
+        * **modality** - The data modality of this project. Currently, Atlas supports either `text` or `embedding` modality projects.
+        * **organization_name** - The name of the organization to create this project under. You must be a member of the organization with appropriate permissions. If not specified, defaults to your user accounts default organization.
+        * **is_public** - Should this project be publicly accessible for viewing (read only). If False, only members of your Nomic organization can view.
+
+        **Returns:** project_id on success.
 
         '''
 
@@ -110,7 +109,7 @@ class AtlasClient:
                 )
             organization_id = organization_id_request.json()['organization_id']
 
-        print(f"Creating project `{project_name}` in organization `{organization_name}`")
+        logger.info(f"Creating project `{project_name}` in organization `{organization_name}`")
 
         response = requests.post(
             self.atlas_api_path + "/v1/project/create",
@@ -132,8 +131,7 @@ class AtlasClient:
         '''
         Retrieves the ID of the current users default organization.
 
-        Returns:
-            The ID of the current users default organization.
+        **Returns:** The ID of the current users default organization
 
         '''
 
@@ -145,8 +143,12 @@ class AtlasClient:
     def _get_project_by_name(self, project_name: str):
         '''
         Retrieves a project that a user owns by name.
-        Args:
-            project_name: the project name
+
+        **Parameters:**
+
+        * **project_name** - The name of the project.
+
+        **Returns:** the project.
 
         Returns:
 
@@ -216,15 +218,15 @@ class AtlasClient:
         '''
         Adds embeddings to an embedding project. Pair each embedding with meta-data to explore your embeddings.
 
-        Args:
-            project_id: The id of the project you are adding emebeddings to.
-            embeddings: An [N,d] numpy array containing the batch of N embeddings to add.
-            data: An [N,] element list of dictionaries containing metadata for each embedding.
-            shard_size: Embeddings are uploaded in parallel by many threads. Adjust the number of embeddings to upload by each worker.
-            num_workers: The number of worker threads to upload embeddings with.
+        **Parameters:**
 
-        Returns:
-            True on success.
+        * **project_id** - The id of the project you are adding embeddings to.
+        * **embeddings** - An [N,d] numpy array containing the batch of N embeddings to add.
+        * **data** - An [N,] element list of dictionaries containing metadata for each embedding.
+        * **shard_size** - Embeddings are uploaded in parallel by many threads. Adjust the number of embeddings to upload by each worker.
+        * **num_workers** - The number of worker threads to upload embeddings with.
+
+        **Returns:** True on success.
         '''
 
         # Each worker currently is to slow beyond a shard_size of 5000
@@ -246,7 +248,7 @@ class AtlasClient:
         # if this method is being called internally, we pass a global progress bar
         close_pbar = False
         if pbar is None:
-            print("Uploading embeddings to Nomic.")
+            logger.info("Uploading embeddings to Nomic.")
             close_pbar = True
             pbar = tqdm(total=int(embeddings.shape[0]) // shard_size)
 
@@ -256,7 +258,7 @@ class AtlasClient:
                 response = future.result()
                 pbar.update(1)
                 if response.status_code != 200:
-                    print(f"Shard upload failed: {response.json()}")
+                    logger.error(f"Shard upload failed: {response.json()}")
                     if 'more datums exceeds your organization limit' in response.json():
                         return False
 
@@ -266,28 +268,28 @@ class AtlasClient:
             pbar.close()
 
         if failed:
-            print(f"Failed to upload {len(failed)*shard_size} datums")
+            logger.warning(f"Failed to upload {len(failed)*shard_size} datums")
         if close_pbar:
             if failed:
-                print("Embedding upload partially succeeded.")
+                logger.warning("Embedding upload partially succeeded.")
             else:
-                print("Embedding upload succeeded.")
+                logger.warning("Embedding upload succeeded.")
 
         return True
 
     def create_index(self, project_id: str, index_name: str, colorable_fields=[]):
         '''
         Creates an index in the specified project
-        Args:
-            project_id: The project id to build the index for.
-            index_name: The name of the index
-            colorable_fields: The project fields you want to be able to color by on the map. Must be a subset of the projects fields.
-
-        Returns:
-            CreateIndexResponse
 
+        **Parameters:**
 
+        * **project_id** - The ID of the project this index is being built under.
+        * **index_name** - The name of the index
+        * **colorable_fields** - The project fields you want to be able to color by on the map. Must be a subset of the projects fields.
+        * **shard_size** - Embeddings are uploaded in parallel by many threads. Adjust the number of embeddings to upload by each worker.
+        * **num_workers** - The number of worker threads to upload embeddings with.
 
+        **Returns:** A link to your map.
         '''
 
         project = self._get_project_by_id(project_id=project_id)
@@ -340,15 +342,16 @@ class AtlasClient:
 
         to_return = {'job_id': job_id, 'index_id': index_id}
         if not projection_id:
-            print("Could not find a projection being built for this index.")
+            logger.warning("Could not find a projection being built for this index.")
         else:
             if self.credentials['tenant'] == 'staging':
                 to_return['map'] = f"https://staging-atlas.nomic.ai/map/{project['id']}/{projection_id}"
             else:
                 to_return['map'] = f"https://atlas.nomic.ai/map/{project['id']}/{projection_id}"
+            logger.info(f"Created map `{index_name}`: {to_return['map']}")
         return CreateIndexResponse(**to_return)
 
-    def add_text(self, project_id: str, data: List[Dict]):
+    def _add_text(self, project_id: str, data: List[Dict]):
         '''
         Adds text to an Atlas text project. Each text datum consists of a keyed dictionary.
         Args:
@@ -371,26 +374,24 @@ class AtlasClient:
         num_workers: int = 10,
         map_name: str = None,
         map_description: str = None,
-        organization_name: str = None
+        organization_name: str = None,
     ):
         '''
         Generates a map of the given embeddings.
 
-        Args:
-            embeddings: An [N,d] numpy array containing the batch of N embeddings to add.
-            data: An [N,] element list of dictionaries containing metadata for each embedding.
-            id_field: Each datums unique id field.
-            colorable_fields: The project fields you want to be able to color by on the map. Must be a subset of the projects fields.
-            is_public: Should this embedding map be public? Private maps can only be accessed by members of your organization.
-            num_workers: number of workers to use when sending data.
-            map_name: A name for your map.
-            map_description: A description for your map.
-            organization_name: The name of the organization to create this project under. You must be a member of the organization with appropriate permissions. If not specified, defaults to your user accounts default organization.
+        **Parameters:**
 
+        * **embeddings** - An [N,d] numpy array containing the batch of N embeddings to add.
+        * **data** - An [N,] element list of dictionaries containing metadata for each embedding.
+        * **id_field** - Each datums unique id field.
+        * **colorable_fields** - The project fields you want to be able to color by on the map. Must be a subset of the projects fields.
+        * **is_public** - Should this embedding map be public? Private maps can only be accessed by members of your organization.
+        * **num_workers** - The number of workers to use when sending data.
+        * **map_name** - A name for your map.
+        * **map_description** - A description for your map.
+        * **organization_name** - *(optional)* The name of the organization to create this project under. You must be a member of the organization with appropriate permissions. If not specified, defaults to your user accounts default organization.
 
-        Returns:
-            CreateIndexResponse
-
+        **Returns:** A link to your map.
         '''
 
         def get_random_name():
@@ -415,7 +416,7 @@ class AtlasClient:
             unique_id_field=id_field,
             modality='embedding',
             is_public=is_public,
-            organization_name=organization_name
+            organization_name=organization_name,
         )
 
         shard_size = 1000
@@ -424,7 +425,7 @@ class AtlasClient:
 
         # sends several requests to allow for threadpool refreshing. Threadpool hogs memory and new ones need to be created.
         MAX_MEMORY_CHUNK = 150000
-        print("Uploading embeddings to Nomic.")
+        logger.info("Uploading embeddings to Nomics' neural database Atlas.")
 
         embeddings = embeddings.astype(np.float16)
 
@@ -439,7 +440,7 @@ class AtlasClient:
                     pbar=pbar,
                 )
 
-        print("Embedding upload succeeded.")
+        logger.info("Embedding upload succeeded.")
 
         response = self.create_index(project_id=project_id, index_name=index_name, colorable_fields=colorable_fields)
 
